@@ -724,6 +724,11 @@ runner.deep_resolve('makeBitString')
 
 # Needed for deparse
 runner.deep_resolve('pg_toupper')
+runner.deep_resolve('makeStringInfo')
+runner.deep_resolve('list_delete_last')
+runner.deep_resolve('list_insert_nth')
+runner.deep_resolve('bms_add_member')
+runner.deep_resolve('bms_is_member')
 
 # Needed for normalize
 runner.deep_resolve('pg_qsort')
@@ -734,6 +739,13 @@ runner.deep_resolve('raw_expression_tree_walker_impl')
 runner.deep_resolve('hash_bytes')
 runner.deep_resolve('MemoryContextAllocExtended')
 
+# Needed for summary
+runner.deep_resolve('makeRangeVarFromNameList')
+runner.deep_resolve('list_sort')
+runner.deep_resolve('pg_mbcharcliplen')
+runner.deep_resolve('pg_mbstrlen')
+runner.deep_resolve('destroyStringInfo')
+
 # Other required functions
 runner.deep_resolve('pg_printf')
 runner.deep_resolve('pg_strncasecmp')
